@@ -1,3 +1,15 @@
+"""
+Low-level interface for controlling Dynamixel smart servos through their SDK.
+
+This module provides comprehensive control of Dynamixel servos including:
+- Direct motor control (position, velocity, torque)
+- Motor configuration (PID, limits, operating modes)
+- Status monitoring (position, load, temperature)
+- Communication management (baudrate, protocol)
+
+The module implements thread-safe operations and supports both Protocol 1.0 and 2.0 servos.
+"""
+
 import ctypes
 import functools
 import logging
@@ -9,6 +21,15 @@ from serial.tools import list_ports
 
 
 def get_logger(name):
+    """
+    Configure and return a logger instance with console output.
+    
+    Args:
+        name (str): Logger name, typically __name__
+        
+    Returns:
+        logging.Logger: Configured logger instance
+    """
     logger = logging.getLogger(name)
     if not logger.handlers:
         # Prevent logging from propagating to the root logger
@@ -18,28 +39,38 @@ def get_logger(name):
         logger.addHandler(console)
         formatter = logging.Formatter(
             "%(asctime)s — %(name)s — %(levelname)s — %(funcName)s:%(lineno)d — %(message)s"
-            # "%(asctime)s — %(levelname)s — %(message)s"
         )
         console.setFormatter(formatter)
     return logger
 
 
 def synchronized(lock):
+    """
+    Decorator for thread-safe method execution.
+    
+    Args:
+        lock: Threading lock object
+        
+    Returns:
+        callable: Decorated function that acquires/releases lock during execution
+    """
     def decorator(wrapped):
         @functools.wraps(wrapped)
         def wrapper(*args, **kwargs):
             with lock:
                 return wrapped(*args, **kwargs)
-
         return wrapper
-
     return decorator
 
 
-# See https://emanual.robotis.com/docs/en/dxl/mx/mx-106-2/ for #
-# appropriate control table values at each address #
 class DXLControlv2:
-    # THESE ARE READ ONLY (FOR STATUS/INFO) #
+    """
+    Control table addresses and settings for Dynamixel Protocol 2.0 servos.
+    
+    Contains memory addresses for all readable/writable servo parameters and
+    configuration settings. Addresses based on MX-106-2 servo specifications.
+    """
+    # READ ONLY ADDRESSES
     DEV_MODEL_NUMBER = 0  # 2 BYTES
     DEV_MODEL_INFO = 2  # 4 BYTES
     DEV_FIRMWARE = 6  # 1 BYTE
@@ -81,6 +112,7 @@ class DXLControlv2:
     DEV_VELCOITY_D = 80  # 2 BYTES
     DEV_POS_P = 84  # 2 BYTES
     DEV_POS_I = 82  # 2 BYTES
+    DEV_POS_D = 80  # 2 BYTES
     BUS_WATCHDOG = 98  # 1 BYTE
     DEV_GOAL_PWM = 100  # 2 BYTES
     DEV_GOAL_CURRENT = 102  # 2 BYTES
@@ -131,6 +163,12 @@ class DXLControlv2:
 
 
 class DXLControlv1:
+    """
+    Control table addresses and settings for Dynamixel Protocol 1.0 servos.
+    
+    Contains memory addresses for all readable/writable servo parameters
+    and configuration settings for older Protocol 1.0 servos.
+    """
     # THESE ARE READ ONLY (FOR STATUS/INFO) #
     DEV_MODEL_NUMBER = 0  # 2 BYTES
     DEV_FIRMWARE = 2  # 1 BYTE
@@ -197,6 +235,7 @@ class DXLControlv1:
 
 
 class DXLDefaults:
+    """Default configuration values for Dynamixel servos."""
     PROTOCOL_VERSION = 2.0
     DEFAULT_BAUDRATE = 1  # maps to 57600
 
@@ -204,6 +243,10 @@ class DXLDefaults:
 
 
 class DXLMotorSDK(DXLDefaults):
+    """
+    SDK for controlling Dynamixel motors via Dynamixel protocol. Provides methods
+    for reading sensor data and setting operating parameters.
+    """
     lock = threading.RLock()
 
     def __init__(
@@ -212,15 +255,29 @@ class DXLMotorSDK(DXLDefaults):
         protocol_version=2.0,
         starting_baudrate=57600,
         target_baudrate=57600,
+        dev=None
     ):
-        devs = list(list_ports.comports())
+        """
+        Initialize the DXLMotorSDK instance with port and protocol settings.
 
-        try:
-            for d in devs:
-                if ("USB" in d.device) or ("usbserial" in d.device):
-                    self.DEVICENAME = d.device
-        except IndexError:
-            raise Exception("No DXL Driver found.")
+        Args:
+            verbose (bool): Whether to log extra info.
+            protocol_version (float): The Dynamixel protocol version.
+            starting_baudrate (int): Initial baudrate for port communication.
+            target_baudrate (int): Target baudrate to which the port will switch.
+            dev (str): Optional device path for the USB interface.
+        """
+        if dev is not None:
+            self.DEVICENAME = dev
+        else:
+            devs = list(list_ports.comports())
+
+            try:
+                for d in devs:
+                    if ("USB" in d.device) or ("usbserial" in d.device):
+                        self.DEVICENAME = d.device
+            except IndexError:
+                raise Exception("No DXL Driver found.")
         self.verbose = verbose
         self.log = get_logger(__name__)
 
@@ -242,6 +299,9 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def connectToDXL(self):
+        """
+        Open the serial port and initialize the PortHandler and PacketHandler.
+        """
         self.portHandler = dxl.PortHandler(self.DEVICENAME)
         if self.portHandler.openPort():
             if self.verbose:
@@ -263,6 +323,15 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def getBaudRate(self, dxl_id):
+        """
+        Read the current baudrate from the servo.
+
+        Args:
+            dxl_id (int): Servo ID to query.
+
+        Returns:
+            int or bool: The servo's baudrate, or False if there's a communication error.
+        """
         (
             dxl_baud_rate,
             dxl_comm_result,
@@ -289,6 +358,15 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def getHardwareError(self, dxl_id):
+        """
+        Get any hardware error status from the servo.
+
+        Args:
+            dxl_id (int): Servo ID to query.
+
+        Returns:
+            int or bool: Hardware error byte, or False on error.
+        """
         (
             dxl_hardware_error,
             dxl_comm_result,
@@ -315,6 +393,15 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def getDriveMode(self, dxl_id):
+        """
+        Retrieve the drive mode setting of a servo.
+
+        Args:
+            dxl_id (int): The servo ID.
+
+        Returns:
+            int or bool: The drive mode value, or False on communication error.
+        """
         (
             dxl_drive_mode,
             dxl_comm_result,
@@ -341,10 +428,25 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def getPortBaudRate(self):
+        """
+        Get the current port baudrate from the PortHandler.
+
+        Returns:
+            int: The current port baudrate.
+        """
         return self.portHandler.getBaudRate()
 
     @synchronized(lock)
     def getDXLID(self, dxl_id):
+        """
+        Read the Dynamixel ID from the servo.
+
+        Args:
+            dxl_id (int): The current servo ID.
+
+        Returns:
+            int or None: The servo's ID or None on error.
+        """
         # Read dxl id #
         (
             dxl_id,
@@ -370,6 +472,13 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def setDXLID(self, dxl_id, target_dxl_id):
+        """
+        Assign a new servo ID.
+
+        Args:
+            dxl_id (int): Current servo ID.
+            target_dxl_id (int): Desired new servo ID.
+        """
         # Read dxl id #
         dxl_comm_result, dxl_error = self.packetHandler.write1ByteTxRx(
             self.portHandler,
@@ -397,6 +506,15 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def getProtocolVersion(self, dxl_id):
+        """
+        Read the Dynamixel protocol version from the servo.
+
+        Args:
+            dxl_id (int): The servo ID.
+
+        Returns:
+            int or None: The servo's protocol version or None on error.
+        """
         # Read dxl id #
         (
             dxl_protocol_version,
@@ -422,6 +540,12 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def clearMultiTurnInfo(self, dxl_id):
+        """
+        Clear the multi-turn information from a servo that supports multi-turn mode.
+
+        Args:
+            dxl_id (int): The servo ID.
+        """
         # Clear Multi-Turn Information #
         dxl_comm_result, dxl_error = self.packetHandler.clearMultiTurn(
             self.portHandler, dxl_id
@@ -441,6 +565,15 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def getPosition(self, dxl_id):
+        """
+        Read the current position (encoder value) of the servo.
+
+        Args:
+            dxl_id (int): The servo ID.
+
+        Returns:
+            int or None: The present position or None on error.
+        """
         # Read present position #
         if self.PROTOCOL_VERSION == 2.0:
             (
@@ -475,6 +608,15 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def getMaxTorque(self, dxl_id):
+        """
+        Get the maximum torque setting for protocol 1.0 servos.
+
+        Args:
+            dxl_id (int): Servo ID to query.
+
+        Returns:
+            int or None: The maximum torque value, or None on error or if protocol 2.0 is used.
+        """
         # Read present max torque #
         if self.PROTOCOL_VERSION == 2.0:
             self.log.info(
@@ -506,6 +648,15 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def getMovingThreshold(self, dxl_id):
+        """
+        Retrieve the servo's moving threshold in RPM.
+
+        Args:
+            dxl_id (int): The servo ID.
+
+        Returns:
+            float or None: The moving threshold in RPM or None on error.
+        """
         # Read present position #
         (
             dxl_moving_threshold,
@@ -531,6 +682,15 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def getGoalPosition(self, dxl_id):
+        """
+        Read the servo's goal position (desired position).
+
+        Args:
+            dxl_id (int): The servo ID.
+
+        Returns:
+            int or None: The goal position or None on error.
+        """
         # Read present position #
         if self.PROTOCOL_VERSION == 2.0:
             (
@@ -565,6 +725,15 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def getVelocity(self, dxl_id):
+        """
+        Get the current velocity of the servo in RPM.
+
+        Args:
+            dxl_id (int): The servo ID.
+
+        Returns:
+            float or None: The current velocity in RPM or None on error.
+        """
         # Read present position #
         if self.PROTOCOL_VERSION == 2.0:
             (
@@ -606,6 +775,15 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def getGoalVelocity(self, dxl_id):
+        """
+        Get the servo's goal velocity in RPM.
+
+        Args:
+            dxl_id (int): The servo ID.
+
+        Returns:
+            float or None: The goal velocity in RPM or None on error.
+        """
         # Read present velocity #
         if self.PROTOCOL_VERSION == 2.0:
             (
@@ -643,6 +821,15 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def getVelocityProfile(self, dxl_id):
+        """
+        Retrieve the velocity profile setting of the servo.
+
+        Args:
+            dxl_id (int): The servo ID.
+
+        Returns:
+            float or None: The velocity profile in RPM or None on error.
+        """
         # Read velocity profile #
         (
             dxl_velocity_profile,
@@ -668,6 +855,15 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def getAccelerationProfile(self, dxl_id):
+        """
+        Get the acceleration profile value of the servo.
+
+        Args:
+            dxl_id (int): The servo ID.
+
+        Returns:
+            int or None: The acceleration profile value or None on error.
+        """
         # Read velocity profile #
         (
             dxl_acceleration_profile,
@@ -694,6 +890,15 @@ class DXLMotorSDK(DXLDefaults):
     # Ranges from 0-100, scale is degrees Celsius #
     @synchronized(lock)
     def getTemperature(self, dxl_id):
+        """
+        Read the servo's temperature in Celsius (0-100 range).
+
+        Args:
+            dxl_id (int): The servo ID.
+
+        Returns:
+            int or None: The current temperature or None on error.
+        """
         # Read present position #
         (
             dxl_current_temp,
@@ -719,6 +924,15 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def getVoltage(self, dxl_id):
+        """
+        Get the servo's supply voltage in Volts.
+
+        Args:
+            dxl_id (int): The servo ID.
+
+        Returns:
+            float or None: The voltage or None on error.
+        """
         # Read present position #
         (
             dxl_current_voltage,
@@ -746,6 +960,15 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def getCurrent(self, dxl_id):
+        """
+        Read the servo's current draw in Amps (approx).
+
+        Args:
+            dxl_id (int): The servo ID.
+
+        Returns:
+            float or None: The current in Amps or None on error.
+        """
         current_unit = 3.36e-3  # A
         # Read present position #
         (
@@ -772,6 +995,15 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def getOperatingMode(self, dxl_id):
+        """
+        Retrieve the operating mode description of the servo.
+
+        Args:
+            dxl_id (int): The servo ID.
+
+        Returns:
+            str or None: The operating mode description or None on error.
+        """
         # Read operating mode #
         (
             dxl_current_mode,
@@ -797,6 +1029,15 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def getTorqueStatus(self, dxl_id):
+        """
+        Read the current torque enable status of the servo.
+
+        Args:
+            dxl_id (int): The servo ID.
+
+        Returns:
+            int or None: The torque status (0 or 1) or None on error.
+        """
         # Read Dynamixel Torque Status #
         (
             torque_status,
@@ -823,7 +1064,15 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def getMovingStatus(self, dxl_id):
-        # Read Dynamixel Torque Status #
+        """
+        Check if the servo is currently moving.
+
+        Args:
+            dxl_id (int): The servo ID.
+
+        Returns:
+            bool: True if moving, False otherwise.
+        """
         (
             moving_status,
             dxl_comm_result,
@@ -841,14 +1090,20 @@ class DXLMotorSDK(DXLDefaults):
                     "%s"
                     % self.packetHandler.getRxPacketError(dxl_error)
                 )
-        # else:
-        #     if self.verbose:
-        #         self.log.info(f"DXL moving status is:{moving_status}")
 
         return bool(moving_status)
 
     @synchronized(lock)
     def getCWAngleLimit(self, dxl_id):
+        """
+        Read the servo's clockwise angle limit.
+
+        Args:
+            dxl_id (int): The servo ID.
+
+        Returns:
+            int or bool: The CW angle limit, or False on error.
+        """
         (
             cw_angle_limit,
             dxl_comm_result,
@@ -875,6 +1130,15 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def getCCWAngleLimit(self, dxl_id):
+        """
+        Read the servo's counterclockwise angle limit.
+
+        Args:
+            dxl_id (int): The servo ID.
+
+        Returns:
+            int or bool: The CCW angle limit, or False on error.
+        """
         (
             ccw_angle_limit,
             dxl_comm_result,
@@ -901,6 +1165,15 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def getVelocityLimit(self, dxl_id):
+        """
+        Retrieve the velocity limit in RPM.
+
+        Args:
+            dxl_id (int): The servo ID.
+
+        Returns:
+            float or None: The velocity limit in RPM or None on error.
+        """
         (
             dxl_velocity_limit,
             dxl_comm_result,
@@ -928,6 +1201,15 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def getPosition_P(self, dxlid):
+        """
+        Get the P gain for position control.
+
+        Args:
+            dxlid (int): The servo ID.
+
+        Returns:
+            int: The P value for position control.
+        """
         (
             position_p,
             dxl_comm_result,
@@ -954,6 +1236,15 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def getPosition_I(self, dxlid):
+        """
+        Get the I gain for position control.
+
+        Args:
+            dxlid (int): The servo ID.
+
+        Returns:
+            int: The I value for position control.
+        """
         (
             position_i,
             dxl_comm_result,
@@ -978,11 +1269,53 @@ class DXLMotorSDK(DXLDefaults):
                 )
         return ctypes.c_int16(position_i).value
 
+    @synchronized(lock)
+    def getPosition_D(self, dxlid):
+        """
+        Get the D gain for position control.
+
+        Args:
+            dxlid (int): The servo ID.
+
+        Returns:
+            int: The D value for position control.
+        """
+        (
+            position_d,
+            dxl_comm_result,
+            dxl_error,
+        ) = self.packetHandler.read2ByteTxRx(
+            self.portHandler, dxlid, self.control.DEV_POS_D
+        )
+        if dxl_comm_result != dxl.COMM_SUCCESS:
+            self.log.info(
+                "%s" % self.packetHandler.getTxRxResult(dxl_comm_result)
+            )
+        elif dxl_error != 0:
+            if self.verbose:
+                self.log.info(
+                    "%s"
+                    % self.packetHandler.getRxPacketError(dxl_error)
+                )
+        else:
+            if self.verbose:
+                self.log.debug(
+                    f"Position PID, I = : {ctypes.c_int16(position_d).value}"
+                )
+        return ctypes.c_int16(position_d).value
+
     ############################
     # BEGINNING OF SET METHODS #
     ############################
     @synchronized(lock)
     def setVelocityLimit(self, dxl_id, velocity_limit_rpm):
+        """
+        Write a new velocity limit to the servo in RPM.
+
+        Args:
+            dxl_id (int): The servo ID.
+            velocity_limit_rpm (float): Desired velocity limit in RPM.
+        """
         # Write goal position #
         velocity_limit_value = self.convertRPM2Value(velocity_limit_rpm)
         dxl_comm_result, dxl_error = self.packetHandler.write4ByteTxRx(
@@ -1011,6 +1344,13 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def setCWAngleLimit(self, dxl_id, cw_angle_limit):
+        """
+        Define the servo's clockwise angle limit.
+
+        Args:
+            dxl_id (int): The servo ID.
+            cw_angle_limit (int): The CW angle limit value.
+        """
         dxl_comm_result, dxl_error = self.packetHandler.write2ByteTxRx(
             self.portHandler,
             dxl_id,
@@ -1037,6 +1377,13 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def setCCWAngleLimit(self, dxl_id, ccw_angle_limit):
+        """
+        Define the servo's counterclockwise angle limit.
+
+        Args:
+            dxl_id (int): The servo ID.
+            ccw_angle_limit (int): The CCW angle limit value.
+        """
         dxl_comm_result, dxl_error = self.packetHandler.write2ByteTxRx(
             self.portHandler,
             dxl_id,
@@ -1070,14 +1417,20 @@ class DXLMotorSDK(DXLDefaults):
         starting_baudrate=57600,
         target_baudrate=9600,
     ):
-        # Set port baudrate #
+        """
+        Change the baudrate of a servo from a starting baudrate to a target baudrate.
+
+        Args:
+            dxl_id (int): The servo ID to configure.
+            starting_baudrate (int): Current known baudrate on the servo.
+            target_baudrate (int): Target baudrate to set on the servo.
+        """
         if self.portHandler.setBaudRate(starting_baudrate):
             if self.verbose:
                 self.log.info(
                     f"Baudrate on PORT set to {starting_baudrate}"
                 )
 
-            # Next set baudrate for DXL #
             (
                 dxl_comm_result,
                 dxl_error,
@@ -1102,7 +1455,6 @@ class DXLMotorSDK(DXLDefaults):
                         % self.packetHandler.getRxPacketError(dxl_error)
                     )
 
-            # finally update port baudrate #
             if self.portHandler.setBaudRate(target_baudrate):
                 if self.verbose:
                     self.log.info(
@@ -1112,10 +1464,15 @@ class DXLMotorSDK(DXLDefaults):
             if self.verbose:
                 self.log.info("Failed to change the baudrate")
 
-    # default = 0, reversed = 1. for the base tracking drive you want to be in reversed mode
     @synchronized(lock)
     def setDXLDriveMode(self, dxl_id, mode=0):
-        # Next set baudrate for DXL #
+        """
+        Set the drive mode on a Dynamixel.
+
+        Args:
+            dxl_id (int): The servo ID.
+            mode (int): Drive mode value to set.
+        """
         dxl_comm_result, dxl_error = self.packetHandler.write1ByteTxRx(
             self.portHandler,
             dxl_id,
@@ -1140,7 +1497,13 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def setOperatingMode(self, dxl_id, mode):
-        # Set operating mode #
+        """
+        Set the servo's operating mode, e.g. position, velocity, etc.
+
+        Args:
+            dxl_id (int): The servo ID.
+            mode (int): Operating mode value to set.
+        """
         dxl_comm_result, dxl_error = self.packetHandler.write1ByteTxRx(
             self.portHandler,
             dxl_id,
@@ -1167,9 +1530,14 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def setEnableTorque(self, dxl_id):
+        """
+        Enable torque on the specified servo.
+
+        Args:
+            dxl_id (int): The servo ID.
+        """
         self.enable_torque = True
 
-        # Enable Dynamixel Torque #
         dxl_comm_result, dxl_error = self.packetHandler.write1ByteTxRx(
             self.portHandler,
             dxl_id,
@@ -1192,9 +1560,14 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def setDisableTorque(self, dxl_id):
+        """
+        Disable torque on the specified servo.
+
+        Args:
+            dxl_id (int): The servo ID.
+        """
         self.enable_torque = False
 
-        # Disable Dynamixel Torque
         dxl_comm_result, dxl_error = self.packetHandler.write1ByteTxRx(
             self.portHandler,
             dxl_id,
@@ -1217,10 +1590,15 @@ class DXLMotorSDK(DXLDefaults):
             if self.verbose:
                 self.log.debug("Torque was disabled")
 
-    # goal_pos is an encoder value which is an int
-    # range in wheel mode: -1,048,575 -> 1,048,575
     @synchronized(lock)
     def setGoalPosition(self, dxl_id, goal_pos):
+        """
+        Command the servo to move to a specific position.
+
+        Args:
+            dxl_id (int): The servo ID.
+            goal_pos (int): Desired position value.
+        """
         # Write goal position #
         if self.PROTOCOL_VERSION == 2.0:
             (
@@ -1254,15 +1632,17 @@ class DXLMotorSDK(DXLDefaults):
                     "%s"
                     % self.packetHandler.getRxPacketError(dxl_error)
                 )
-        # else:
-        #     if self.verbose:
-        #         self.log.info(f"Goal position set to {goal_pos}")
 
-    # range from 0 -> 1023, linear increase corresponds to %-age of total torque to be used
     @synchronized(lock)
     def setMaxTorque(self, dxl_id, goal_max_torque):
+        """
+        Set the maximum torque for protocol 1.0 servos.
+
+        Args:
+            dxl_id (int): The servo ID.
+            goal_max_torque (int): The target max torque value.
+        """
         goal_max_torque = int(goal_max_torque)
-        # Write goal position #
         if self.PROTOCOL_VERSION == 2.0:
             self.log.info(
                 "This method is not yet supported for protocol 2.0!"
@@ -1297,7 +1677,13 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def setMovingThreshold(self, dxl_id, target_moving_threshold_rpm):
-        # Write goal position #
+        """
+        Configure the servo's moving threshold in RPM.
+
+        Args:
+            dxl_id (int): The servo ID.
+            target_moving_threshold_rpm (float): Threshold in RPM.
+        """
         target_moving_threshold_value = self.convertRPM2Value(
             target_moving_threshold_rpm
         )
@@ -1328,6 +1714,13 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def setPosition_P(self, dxlid, p_value):
+        """
+        Set the P gain for position control.
+
+        Args:
+            dxlid (int): The servo ID.
+            p_value (int): Desired P gain value.
+        """
         dxl_comm_result, dxl_error = self.packetHandler.write2ByteTxRx(
             self.portHandler, dxlid, self.control.DEV_POS_P, p_value
         )
@@ -1346,6 +1739,13 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def setPosition_I(self, dxlid, i_value):
+        """
+        Set the I gain for position control.
+
+        Args:
+            dxlid (int): The servo ID.
+            i_value (int): Desired I gain value.
+        """
         dxl_comm_result, dxl_error = self.packetHandler.write2ByteTxRx(
             self.portHandler, dxlid, self.control.DEV_POS_I, i_value
         )
@@ -1362,9 +1762,40 @@ class DXLMotorSDK(DXLDefaults):
                     % self.packetHandler.getRxPacketError(dxl_error)
                 )
 
+    @synchronized(lock)
+    def setPosition_D(self, dxlid, d_value):
+        """
+        Set the D gain for position control.
+
+        Args:
+            dxlid (int): The servo ID.
+            d_value (int): Desired D gain value.
+        """
+        dxl_comm_result, dxl_error = self.packetHandler.write2ByteTxRx(
+            self.portHandler, dxlid, self.control.DEV_POS_D, d_value
+        )
+        if dxl_comm_result != dxl.COMM_SUCCESS:
+            if self.verbose:
+                self.log.info(
+                    "%s"
+                    % self.packetHandler.getTxRxResult(dxl_comm_result)
+                )
+        if dxl_error != 0:
+            if self.verbose:
+                self.log.info(
+                    "%s"
+                    % self.packetHandler.getRxPacketError(dxl_error)
+                )
+
     @synchronized(lock)
     def setVelocity(self, dxl_id, rpm_goal):
-        # Write goal velocity #
+        """
+        Set the goal velocity in RPM for the servo.
+
+        Args:
+            dxl_id (int): The servo ID.
+            rpm_goal (float): Desired velocity in RPM.
+        """
         GOAL_VELOCITY = self.convertRPM2Value(rpm_goal)
         if self.PROTOCOL_VERSION == 2.0:
             (
@@ -1402,6 +1833,12 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def rebootMotor(self, dxl_id):
+        """
+        Reboot the specified servo. This can clear certain errors.
+
+        Args:
+            dxl_id (int): The servo ID to reboot.
+        """
         dxl_comm_result, dxl_error = self.packetHandler.reboot(
             self.portHandler, dxl_id
         )
@@ -1426,7 +1863,15 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def getLoad(self, dxl_id):
-        # Read Current Load Value #
+        """
+        Get the load percentage from the servo.
+
+        Args:
+            dxl_id (int): The servo ID.
+
+        Returns:
+            float: Load as a percentage (signed).
+        """
         (
             current_load,
             dxl_comm_result,
@@ -1453,10 +1898,16 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def setVelocityProfile(self, dxl_id, velocity_profile_rpm):
+        """
+        Set the velocity profile value in RPM for protocol 2.0 servos.
+
+        Args:
+            dxl_id (int): The servo ID.
+            velocity_profile_rpm (float): Velocity profile in RPM.
+        """
         velocity_profile_value = self.convertRPM2Value(
             velocity_profile_rpm
         )
-        # Write goal velocity #
         if self.PROTOCOL_VERSION == 2.0:
             (
                 dxl_comm_result,
@@ -1482,7 +1933,13 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def setAccelerationProfile(self, dxl_id, acceleration_profile):
-        # Write goal velocity #
+        """
+        Set the acceleration profile for protocol 2.0 servos.
+
+        Args:
+            dxl_id (int): The servo ID.
+            acceleration_profile (int): The acceleration value to set.
+        """
         if self.PROTOCOL_VERSION == 2.0:
             (
                 dxl_comm_result,
@@ -1508,6 +1965,15 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def convertRPM2Value(self, rpm):
+        """
+        Convert an RPM value to the register value based on the servo's RPM unit.
+
+        Args:
+            rpm (float): The RPM to convert.
+
+        Returns:
+            int: The register value corresponding to the RPM.
+        """
         # For conversions #
         # RPM2RADPERSEC = 0.104719755  # this is 2pi / 60 seconds
         RPM_UNIT = self.RPM_UNIT
@@ -1526,6 +1992,15 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def convertValue2RPM(self, value):
+        """
+        Convert a servo register value to an RPM value based on the servo's RPM unit.
+
+        Args:
+            value (int): The register value.
+
+        Returns:
+            float: The RPM corresponding to the register value.
+        """
         # For conversions #
         # RPM2RADPERSEC = 0.104719755  # this is 2pi / 60 seconds
         RPM_UNIT = 0.229  # 0.229 is from DXL e-manual
@@ -1539,33 +2014,15 @@ class DXLMotorSDK(DXLDefaults):
 
         return rpm
 
-    # @synchronized(lock)
-    def moveDeltaPosition(self, dxl_id, posDelta, speed=2):
-        goal_position = self.getPosition(dxl_id) + posDelta
-        moving_speed = speed
-
-        if posDelta < 0:
-            movingDirection = -1
-        else:
-            movingDirection = 1
-
-        self.setEnableTorque(dxl_id)
-        self.setVelocity(dxl_id, movingDirection * moving_speed)
-        # current_position = self.getPosition(dxl_id)
-        # max_load_in_move = -1
-        while (
-            abs(self.getPosition(dxl_id) - goal_position)
-            > self.encoder_margin
-        ):
-            time.sleep(0.0001)
-        self.setDisableTorque(dxl_id)
-        # self.setVelocity(dxl_id, 0)
-        time.sleep(0.1)
-
-        # print(f"\n Max Load: {max_load_in_move}")
-
     @synchronized(lock)
     def setHomingOffset(self, dxl_id, offset):
+        """
+        Set the homing offset for when the servo is in multi-turn or extended position control mode.
+
+        Args:
+            dxl_id (int): The servo ID.
+            offset (int): The offset value.
+        """
         # Write goal velocity #
         dxl_comm_result, dxl_error = self.packetHandler.write4ByteTxRx(
             self.portHandler,
@@ -1588,6 +2045,12 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def setLEDOn(self, dxl_id):
+        """
+        Turn the servo's LED on (supported models only).
+
+        Args:
+            dxl_id (int): The servo ID.
+        """
         dxl_comm_result, dxl_error = self.packetHandler.write2ByteTxRx(
             self.portHandler, dxl_id, self.control.DEV_TOGGLE_LED, 1
         )
@@ -1606,6 +2069,12 @@ class DXLMotorSDK(DXLDefaults):
 
     @synchronized(lock)
     def setLEDOff(self, dxl_id):
+        """
+        Turn the servo's LED off (supported models only).
+
+        Args:
+            dxl_id (int): The servo ID.
+        """
         dxl_comm_result, dxl_error = self.packetHandler.write2ByteTxRx(
             self.portHandler, dxl_id, self.control.DEV_TOGGLE_LED, 0
         )
@@ -1620,10 +2089,4 @@ class DXLMotorSDK(DXLDefaults):
                 self.log.info(
                     "%s"
                     % self.packetHandler.getRxPacketError(dxl_error)
-                )
-
-    def moveTurns(self, dxl_id, turns, speed=4):
-        delta_encoder_clicks = turns * 4096.0
-        self.moveDeltaPosition(
-            dxl_id, delta_encoder_clicks, speed=speed
-        )
+                )
\ No newline at end of file
